from setuptools import setup


setup(
    name='pyretry',
    version="1.0.0",
    description='Separate your retry logic from your business logic',
    author='Bob Renwick',
    author_email='bob.renwick@gmail.com',
    url='https://github.com/bobbyrenwick/pyretry',
    packages=['pyretry'],
    classifiers=(
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
    ),
)



import time

from functools import wraps


def retry(exceptions_to_catch, num_retries=5, timeout=0, hook=None):
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            timeout_is_func = hasattr(timeout, '__call__')

            for i in range(num_retries + 1):
                attempt_number = i + 1

                try:
                    return func(*args, **kwargs)
                except exceptions_to_catch as e:
                    if i == num_retries:
                        raise e

                    t = timeout(attempt_number) if timeout_is_func else timeout

                    if hook is not None:
                        hook(e, attempt_number, t)

                    if timeout:
                        time.sleep(t)

        return wrapper
    return decorator



import unittest

import mock

from .pyretry import retry


class RetryableError(Exception):
    pass


class DifferentRetryableError(Exception):
    pass


class UnretryableError(Exception):
    pass


class TestPyretry(unittest.TestCase):

    def setUp(self):
        self.counter = 0

    def test_none_failing_function(self):

        @retry(RetryableError)
        def no_errors():
            self.counter += 1
            return 'succeeded'

        r = no_errors()

        self.assertEqual(r, 'succeeded')
        self.assertEqual(self.counter, 1)

    def test_retries_once(self):
        @retry(RetryableError)
        def fails_once():
            self.counter += 1

            if self.counter == 1:
                raise RetryableError
            else:
                return 'succeeded'

        r = fails_once()
        self.assertEqual(r, 'succeeded')
        self.assertEqual(self.counter, 2)

    def test_limit_is_reached(self):
        @retry(RetryableError, num_retries=5)
        def always_fails():
            self.counter += 1
            raise RetryableError

        self.assertRaises(RetryableError, always_fails)
        self.assertEqual(self.counter, 6)

    def test_other_exception_not_caught(self):
        @retry(RetryableError)
        def raises_unretryable():
            self.counter += 1
            raise UnretryableError

        self.assertRaises(UnretryableError, raises_unretryable)
        self.assertEqual(self.counter, 1)

    def test_multiple_exception(self):
        @retry((RetryableError, DifferentRetryableError))
        def raises_multiple_exceptions():
            self.counter += 1

            if self.counter == 1:
                raise RetryableError
            if self.counter == 2:
                raise DifferentRetryableError
            if self.counter == 3:
                return 'succeeded'

        r = raises_multiple_exceptions()
        self.assertEqual(r, 'succeeded')
        self.assertEqual(self.counter, 3)

    def test_timeout_as_float(self):
        @retry(RetryableError, timeout=0.1)
        def fails_twice():
            self.counter += 1

            if self.counter == 3:
                return 'succeeded'
            else:
                raise RetryableError

        r = fails_twice()
        self.assertEqual(r, 'succeeded')
        self.assertEqual(self.counter, 3)

    def test_timeout_as_function(self):
        def side_effect(retry_number):
            return retry_number / 100.0

        timeout_calc = mock.MagicMock()
        timeout_calc.side_effect = side_effect

        @retry(RetryableError, num_retries=5, timeout=timeout_calc)
        def always_fails():
            self.counter += 1
            raise RetryableError

        self.assertRaises(RetryableError, always_fails)
        self.assertEqual(self.counter, 6)

        expected = [mock.call(i + 1) for i in range(5)]
        self.assertEqual(timeout_calc.call_args_list, expected)

    def test_hook_is_called(self):
        hook = mock.MagicMock()
        timeout = 0.1

        @retry(RetryableError, num_retries=5, timeout=timeout, hook=hook)
        def always_fails():
            self.counter += 1
            raise RetryableError

        self.assertRaises(RetryableError, always_fails)
        self.assertEqual(self.counter, 6)

        for i, call in enumerate(hook.call_args_list):
            self.assertTrue(isinstance(call[0][0], RetryableError))
            self.assertEqual(call[0][1], i + 1)
            self.assertEqual(call[0][2], timeout)
